(function() {
    var BASE_FIELDS, DBField, o;
    var __indexOf = Array.prototype.indexOf || function(item) {
      for (var i = 0, l = this.length; i < l; i++) {
        if (this[i] === item) return i;
      }
      return -1;
    };
    DBField = function(base_field, connection, sql_type, extra, map) {
        this.field = base_field;
        this.connection = connection;
        this._sql_type = sql_type;
        this._extra = extra;
        this.map = map;
        return this;
    };
    DBField.prototype.defer_fk_constraint = function() {
        return false;
    };
    DBField.prototype.related_table = function() {
        return this.field.related._meta.db_table;
    };
    DBField.prototype.get_related_type = function() {
        var force_type, related_field, related_type;
        related_field = this.field.related._schema.get_field_by_name(this.field.to_field);
        force_type = related_field.db_type === 'id' ? 'integer' : null;
        related_type = this.connection.negotiate_type(related_field, force_type);
        return related_type.sql_type();
    };
    DBField.prototype.sql_type = function() {
        if (this._sql_type instanceof Function) {
            return this._sql_type.call(this);
        } else {
            return this._sql_type;
        }
    };
    DBField.prototype.extra = function() {
        var extra;
        extra = this._extra;
        if (extra instanceof Function) {
            extra = extra.call(this);
        }
        if (extra != null) {
            return extra;
        } else {
            return '';
        }
    };
    DBField.prototype.quote = function(what) {
        return this.connection.quote(what);
    };
    DBField.prototype.framing = function() {
        return "" + (this.quote(this.field.db_field())) + " \n" + (this.sql_type()) + "\n" + (this.field.nullable ? 'NULL DEFAULT NULL' : 'NOT NULL') + " \n" + (this.field.unique ? 'UNIQUE' : '') + " \n" + (this.field.primary_key ? 'PRIMARY KEY' : '') + "\n" + (this.extra());
    };
    DBField.prototype.get_real_field = function(field_name) {
        var BaseModel, real_field;
        BaseModel = require('../models').BaseModel;
        if (!this.field.related.__locked__) {
            BaseModel.lock(this.field.related);
        }
        real_field = this.field.related._schema.get_field_by_name(field_name);
        return real_field.db_field();
    };
    DBField.prototype.contribute_to_table = function(fields, pending_constraints, visited) {
        var _ref;
        if (this.field.related && !(_ref = this.field.related, __indexOf.call(visited, _ref) >= 0)) {
            this.defer_fk_constraint = function() {
                return true;
            };
            pending_constraints.push(("ALTER TABLE " + this.field.model._meta.db_table + " \nADD CONSTRAINT " + (this.field.db_field()) + "_refs_" + (this.get_real_field(this.field.to_field)) + "\nFOREIGN KEY (" + (this.field.db_field()) + ") \nREFERENCES " + (this.related_table()) + " (" + (this.get_real_field(this.field.to_field)) + ") ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED").replace(/\n/g, ' '));
        }
        return fields.push(this.framing().replace(/\n/g, ' '));
    };
    o = function(extra, sql, map) {
        return function(base_field, connection) {
            var field_class;
            field_class = connection.db_field_type();
            return new field_class(base_field, connection, sql, extra, map || {
                js_to_db: function(val) {
                    return val;
                },
                db_to_js: function(val) {
                    return val;
                }
            });
        };
    };
    BASE_FIELDS = {
        date: o(null, "DATE"),
        datetime: o(null, "DATETIME"),
        varchar: o(null, function() {
            return "VARCHAR(" + this.field.max_length + ")";
        }),
        integer: o(null, "INTEGER"),
        text: o(null, "TEXT"),
        id: o((function() {
            return "AUTOINCREMENT";
        }), "INTEGER"),
        fk: o((function() {
            if (!this.defer_fk_constraint()) {
                return "REFERENCES " + (this.related_table()) + " (" + (this.get_real_field(this.field.to_field)) + ") ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED";
            } else {
                return "";
            }
        }), (function() {
            return this.get_related_type(this.connection);
        }))
    };
    exports.BASE_FIELDS = BASE_FIELDS;
    exports.DBField = DBField;
    exports.o = o;
}).call(this);
