(function() {
    var AND, BASE_FIELDS, Connection, DELETE, INSERT, NOT, OR, PGConnection, PGWrapper, SELECT, UPDATE, comparisons, pg, _ref;
    Connection = require('../../connection').Connection;
    _ref = require('../../constants'), AND = _ref.AND, OR = _ref.OR, NOT = _ref.NOT, SELECT = _ref.SELECT, INSERT = _ref.INSERT, UPDATE = _ref.UPDATE, DELETE = _ref.DELETE;
    BASE_FIELDS = require('./fields').BASE_FIELDS;
    comparisons = require('./comparisons').comparisons;
    try {
        pg = require('pg');
    } catch (err) {
        try {
            pg = require('pg/lib');
        } catch (err) {
            throw new Error('``pg`` must be installed to use the postgres backend.');
        }
    }
    PGWrapper = function(client) {
        this.client = client;
        return this;
    };
    PGWrapper.prototype.execute = function(sql, values, mode, model, ready) {
        if (mode === INSERT) {
            sql += ' RETURNING *';
        }
        return this.client.query(sql, values, function(err, data) {
            if (mode === SELECT || mode === INSERT) {
                return ready(err, !err ? data.rows : null);
            } else {
                return ready(err, data);
            }
        });
    };
    PGWrapper.prototype.close = function(ready) {
        this.client.end();
        if (ready instanceof Function) {
            return ready();
        }
    };
    PGConnection = function(metadata) {
        this.metadata = metadata;
        return this;
    };
    PGConnection.prototype = new Connection;
    PGConnection.prototype.constraint = function(constraint) {
        return constraint;
    };
    PGConnection.prototype.quote = function(what) {
        return "\"" + (what.replace(/\"/g, '\"\"')) + "\"";
    };
    PGConnection.prototype.comparison = function(name) {
        return comparisons[name];
    };
    PGConnection.prototype.negotiate_type = function(field, force_type) {
        return BASE_FIELDS[force_type || field.db_type](field, this);
    };
    PGConnection.prototype.close = function(ready) {
        return this.client(function(client) {
            return client.close(ready);
        });
    };
    PGConnection.prototype.drop_table = function(model) {
        return "" + (Connection.prototype.drop_table.call(this, model)) + " CASCADE";
    };
    PGConnection.prototype.get_client = function(ready) {
        var config;
        config = {
            user: this.metadata.user,
            password: this.metadata.password,
            database: this.metadata.name,
            host: this.metadata.host || 'localhost',
            port: this.metadata.port || 5432
        };
        return pg.connect(config, function(err, client) {
            if (err) {
                throw err;
            }
            return ready(new PGWrapper(client));
        });
    };
    exports.Connection = PGConnection;
}).call(this);
