(function() {
    var AND, BASE_FIELDS, Connection, DBField, DELETE, INSERT, NOT, OR, SELECT, SQLiteConnection, SQLiteWrapper, UPDATE, comparisons, sqlite3, _ref, _ref2;
    Connection = require('../../connection').Connection;
    _ref = require('../../constants'), AND = _ref.AND, OR = _ref.OR, NOT = _ref.NOT, SELECT = _ref.SELECT, INSERT = _ref.INSERT, UPDATE = _ref.UPDATE, DELETE = _ref.DELETE;
    _ref2 = require('./fields'), BASE_FIELDS = _ref2.BASE_FIELDS, DBField = _ref2.DBField;
    comparisons = require('./comparisons').comparisons;
    try {
        sqlite3 = require('sqlite3');
    } catch (err) {
        try {
            sqlite3 = require('sqlite3/lib');
        } catch (err) {
            throw new Error('``sqlite3`` must be installed to use the sqlite3 backend.');
        }
    }
    SQLiteWrapper = function(client) {
        this.client = client;
        return this;
    };
    SQLiteWrapper.prototype.execute = function(sql, values, mode, model, ready) {
        var args, client, stmt, stmt_ready;
        client = this.client;
        stmt = this.client.prepare(sql);
        stmt_ready = function(err, rows) {
            var lastID;
            if (err) {
                return ready(err, null);
            } else {
                if (mode === INSERT) {
                    lastID = this.lastID;
                    return client.all("SELECT * FROM " + model._meta.db_table + " WHERE \"_ROWID_\" = $1", lastID, function(err, data) {
                        return ready(err, data);
                    });
                } else {
                    return ready(err, rows);
                }
            }
        };
        args = values.slice().concat(stmt_ready);
        stmt[mode === INSERT ? 'run' : 'all'].apply(stmt, args);
        return stmt.on('error', function(err) {
            return ready(err);
        });
    };
    SQLiteWrapper.prototype.close = function(ready) {
        this.client.end();
        if (ready instanceof Function) {
            return ready();
        }
    };
    SQLiteConnection = function(metadata) {
        this.metadata = metadata;
        return this;
    };
    SQLiteConnection.prototype = new Connection;
    SQLiteConnection.prototype.constraint = function(constraint) {
        return constraint;
    };
    SQLiteConnection.prototype.quote = function(what) {
        return "\"" + (what.replace(/\"/g, '\"\"')) + "\"";
    };
    SQLiteConnection.prototype.negotiate_type = function(field, force_type) {
        return BASE_FIELDS[force_type || field.db_type](field, this);
    };
    SQLiteConnection.prototype.comparison = function(type) {
        return comparisons[type];
    };
    SQLiteConnection.prototype.close = function(ready) {
        return this.client(function(client) {
            return client.close(ready);
        });
    };
    SQLiteConnection.prototype.get_client = function(ready) {
        var config, db;
        config = {
            database: this.metadata.name
        };
        return db = new sqlite3.Database(config.database, sqlite3.OPEN_READWRITE | sqlite3.OPEN_CREATE, function(err) {
            if (err) {
                throw err;
            }
            return ready(new SQLiteWrapper(db));
        });
    };
    SQLiteConnection.prototype.db_field_type = function() {
        return DBField;
    };
    exports.Connection = SQLiteConnection;
}).call(this);
