(function() {
    var Connection, EventEmitter;
    var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
    EventEmitter = require('events').EventEmitter;
    Connection = function() {
        return this;
    };
    Connection["default"] = 'default';
    Connection.registry = {};
    Connection.register = function(name, connection_data) {
        return this.registry[name] = connection_data;
    };
    Connection.set_default = function(name) {
        return Connection["default"] = name;
    };
    Connection.get_connection = function(name) {
        var backend, target, _ref;
        name = name || Connection["default"];
        if (name && this.registry[name]) {
            if (Connection.registry[name].connection) {
                return Connection.registry[name].connection;
            } else {
                _ref = Connection.registry[name].backend.split('#'), backend = _ref[0], target = _ref[1];
                backend = require(backend);
                if (!target) {
                    target = 'Connection';
                }
                Connection.registry[name].connection = new backend[target](Connection.registry[name]);
                return Connection.registry[name].connection;
            }
            return Connection.registry[name].connection;
        } else {
            throw new Error("No connection registered under " + name);
        }
    };
    Connection.prototype.client = function(ready) {
        if (!this._client) {
            return this.get_client(__bind(function(client) {
                this._client = client;
                return ready(client);
            }, this));
        } else {
            return ready(this._client);
        }
    };
    Connection.prototype.quote = function(what) {
        return "`" + what + "`";
    };
    Connection.prototype.execute = function(sql, values, mode, model) {
        var ee;
        ee = new EventEmitter;
        this.client(__bind(function() {
            return this._client.execute(sql, values, mode, model, function(err, data) {
                if (err) {
                    err.__sql = sql;
                    err.__values = values;
                }
                if (err) {
                    return ee.emit('error', err);
                } else {
                    return ee.emit('data', data);
                }
            });
        }, this));
        return ee;
    };
    Connection.prototype.drop_table = function(model) {
        return "DROP TABLE " + (this.quote(model._meta.db_table));
    };
    Connection.prototype.close = function(ready) {
        throw new Error('Not implemented in base connection');
    };
    Connection.prototype.constraint = function(constraint) {
        throw new Error('Not implemented in base connection');
    };
    Connection.prototype.negotiate_type = function(field_type) {
        throw new Error('Not implemented in base connection');
    };
    Connection.prototype.comparison = function(comparison) {
        throw new Error('Not implemented in base connection');
    };
    Connection.prototype.db_field_type = function() {
        var DBField;
        DBField = require('./backends/fields').DBField;
        return DBField;
    };
    exports.Connection = Connection;
}).call(this);
