(function() {
    var AutoField, CharField, Count, DateField, DateTimeField, EventEmitter, Field, ForeignKey, IntegerField, Manager, ManyToMany, PositiveIntegerField, ReverseRelation, TextField, exports;
    var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
    Manager = require('./managers').Manager;
    EventEmitter = require('./events').EventEmitter;
    Field = function(kwargs) {
        this.required = kwargs.required, this["default"] = kwargs["default"], this.primary_key = kwargs.primary_key, this.db_index = kwargs.db_index, this.nullable = kwargs.nullable;
        return this;
    };
    Field.prototype.get_prepdb_value = function(value) {
        return value;
    };
    Field.prototype.db_field = function() {
        return this.name;
    };
    Field.prototype.validate_comparison = function(value) {
        return true;
    };
    Field.prototype.validate_value = function(value) {
        return true;
    };
    Field.prototype.apply_value = function(instance, value) {
        return instance[this.name] = value;
    };
    Field.prototype.needs_connection = function() {
        return false;
    };
    Field.prototype.connect = function() {
        return false;
    };
    Field.prototype.create_instance = function(name, model) {
        var clone, _ref;
        clone = Object.create(this);
        _ref = [name, model], clone.name = _ref[0], clone.model = _ref[1];
        return clone;
    };
    DateField = function(kwargs) {
        Field.call(this, kwargs);
        return this;
    };
    DateField.prototype = new Field({});
    DateField.prototype.db_type = 'date';
    DateField.prototype.apply_value = function(instance, value) {
        return instance[this.name] = !isNaN(~~value) ? new Date(value) : Date.parse(value);
    };
    DateField.prototype.validate_comparison = function(value) {
        return !isNaN(Date.parse(value));
    };
    DateField.prototype.validate_value = function(value) {
        if (!(value != null)) {
            return this.nullable;
        } else {
            return !isNaN(Date.parse(value));
        }
    };
    DateField.prototype.get_prepdb_value = function(value) {
        return new Date(Date.parse(value));
    };
    DateTimeField = function(kwargs) {
        DateField.call(this, kwargs);
        return this;
    };
    DateTimeField.prototype = new DateField({});
    DateTimeField.prototype.db_type = 'datetime';
    CharField = function(kwargs) {
        Field.call(this, kwargs);
        this.max_length = kwargs.max_length, this.regex = kwargs.regex;
        if (!this.max_length) {
            this.max_length = 255;
        }
        return this;
    };
    CharField.prototype = new Field({});
    CharField.prototype.db_type = 'varchar';
    CharField.prototype.validate_comparison = function(value) {
        return typeof value === 'string' || (!(typeof val !== "undefined" && val !== null) && this.nullable);
    };
    CharField.prototype.validate_value = function(value) {
        var valid;
        if (!(value != null)) {
            return this.nullable;
        } else {
            valid = value.length < this.max_length;
            if (this.regex) {
                this.regex.lastIndex = 0;
                valid = valid && this.regex.test(value);
            }
            return valid;
        }
    };
    TextField = function(kwargs) {
        Field.call(this, kwargs);
        return this;
    };
    TextField.prototype = new CharField({});
    TextField.prototype.db_type = 'text';
    TextField.prototype.validate_value = function(value) {
        if (!(value != null)) {
            return this.nullable;
        } else {
            return true;
        }
    };
    IntegerField = function(kwargs) {
        Field.call(this, kwargs);
        this.min = kwargs.min, this.max = kwargs.max;
        return this;
    };
    IntegerField.prototype = new Field({});
    IntegerField.prototype.db_type = 'integer';
    IntegerField.prototype.validate_value = function(val) {
        var valid;
        if (!(val != null)) {
            return this.nullable;
        } else {
            valid = typeof val === 'number';
            valid = valid && (this.min ? val > this.min : valid);
            return valid = valid && (this.max ? val < this.max : valid);
        }
    };
    IntegerField.prototype.validate_comparison = function(val) {
        return typeof val === 'number' || (!(val != null) && this.nullable);
    };
    PositiveIntegerField = function(kwargs) {
        kwargs.min = -1;
        IntegerField.call(this, kwargs);
        return this;
    };
    PositiveIntegerField.prototype = new IntegerField({});
    AutoField = function(kwargs) {
        kwargs.primary_key = true;
        PositiveIntegerField.call(this, kwargs);
        return this;
    };
    AutoField.prototype = new PositiveIntegerField({});
    AutoField.prototype.db_type = 'id';
    ReverseRelation = function(name, local_field, remote_field) {
        var _ref;
        this.name = name;
        _ref = [local_field, remote_field], this.local_field = _ref[0], this.remote_field = _ref[1];
        this.related = remote_field.model;
        return this;
    };
    ReverseRelation.prototype = new Field({});
    ReverseRelation.prototype.db_field = function() {
        return null;
    };
    ReverseRelation.prototype.join_struct = function() {
        var join_type, remote_join, reversed_struct, struct;
        remote_join = this.remote_field.join_struct();
        join_type = this.remote_field.join_type();
        reversed_struct = function(struct) {
            var lhs, lhs_field, rhs, rhs_field;
            lhs = struct.lhs, lhs_field = struct.lhs_field, rhs = struct.rhs, rhs_field = struct.rhs_field;
            return {
                lhs: rhs,
                lhs_field: rhs_field,
                rhs: lhs,
                rhs_field: lhs_field,
                join_type: join_type
            };
        };
        return ((function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = remote_join.length; _i < _len; _i++) {
                struct = remote_join[_i];
                _results.push(reversed_struct(struct));
            }
            return _results;
        })()).reverse();
    };
    ForeignKey = function(related, kwargs) {
        PositiveIntegerField.call(this, kwargs);
        this.related_name = kwargs.related_name, this.to_field = kwargs.to_field;
        this.to_field = this.to_field || 'pk';
        this.related = related;
        return this;
    };
    ForeignKey.prototype = new PositiveIntegerField({});
    ForeignKey.prototype.validate_comparison = function(val) {
        return (val instanceof this.related) || (!(val != null) && this.nullable);
    };
    ForeignKey.prototype.validate_value = function(val) {
        var valid;
        if (!(val != null)) {
            return this.nullable;
        } else {
            valid = val instanceof this.related;
            return valid = valid && (val[this.to_field] != null);
        }
    };
    ForeignKey.prototype.db_type = 'fk';
    ForeignKey.prototype.db_field = function() {
        return "" + this.name + "_id";
    };
    ForeignKey.prototype.needs_connection = function() {
        return true;
    };
    ForeignKey.prototype.get_related_name = function() {
        return this.related_name || ("" + (this.model._meta.name.toLowerCase()) + "_set");
    };
    ForeignKey.prototype.apply_value = function(obj, val) {
        obj._fk_cache[this.name] = obj._fk_cache[this.name] || {};
        return obj._fk_cache[this.name].id = val;
    };
    ForeignKey.prototype.join_type = function() {
        return "LEFT";
    };
    ForeignKey.prototype.join_struct = function() {
        return [
            {
                lhs: this.model,
                lhs_field: this,
                rhs: this.related,
                rhs_field: this.related._schema.get_field_by_name(this.to_field),
                join_type: this.join_type()
            }
        ];
    };
    ForeignKey.prototype.get_prepdb_value = function(val) {
        return this.related._schema.get_field_by_name(this.to_field).get_prepdb_value(val[this.to_field]);
    };
    ForeignKey.prototype.connect = function() {
        var db_field, field_name, file, local_mgr, mgr, model, target, to_field, _ref, _ref2;
        if (typeof this.related === 'string') {
            _ref = this.related.split('#'), file = _ref[0], target = _ref[1];
            file = require(file);
            this.related = file[target];
        }
        local_mgr = new Manager(this.related);
        _ref2 = [this.model, this.name, this.db_field, this.to_field], model = _ref2[0], field_name = _ref2[1], db_field = _ref2[2], to_field = _ref2[3];
        this.model.prototype[this.name] = function() {
            var ee, filter, instance;
            if (this._fk_cache[field_name]) {
                if (this._fk_cache[field_name].instance) {
                    instance = this._fk_cache[field_name].instance;
                    ee = new EventEmitter;
                    setTimeout((function() {
                        return ee.emit('data', instance);
                    }), 0);
                    return ee;
                } else if (this._fk_cache[field_name].id) {
                    filter = {};
                    filter[to_field] = this._fk_cache[field_name].id;
                    return local_mgr.get(filter);
                }
            }
        };
        mgr = new Manager(this.model);
        mgr.start_query = function() {
            var base, filter;
            base = Manager.prototype.start_query.call(mgr);
            filter = {};
            filter[field_name] = this.instance;
            return base.filter(filter);
        };
        mgr.get_base_payload = function() {
            var base;
            base = Manager.prototype.get_base_payload.call(this);
            base = base || {};
            base[field_name] = this.instance;
            return base;
        };
        this.model.scope.depend_on(this.related.scope);
        this.related._schema.register_related_field(new ReverseRelation(this.get_related_name(), this.to_field, this));
        return this.related.register_manager(this.get_related_name(), mgr);
    };
    ManyToMany = function(related, kwargs) {
        PositiveIntegerField.call(this, kwargs);
        this.through = kwargs.through, this.related_name = kwargs.related_name;
        this.related = related;
        return this;
    };
    ManyToMany.prototype = new PositiveIntegerField({});
    ManyToMany.prototype.join_type = function() {
        return "INNER";
    };
    ManyToMany.prototype.db_field = function() {
        return null;
    };
    ManyToMany.prototype.needs_connection = function() {
        return true;
    };
    ManyToMany.prototype.get_related_name = function() {
        return this.related_name || ("" + (this.model._meta.name.toLowerCase()) + "_set");
    };
    ManyToMany.prototype.join_struct = function() {
        return [
            {
                lhs: this.model,
                lhs_field: this.model._schema.get_field_by_name('pk'),
                rhs: this.through,
                rhs_field: this.through_local_field,
                join_type: this.join_type()
            }, {
                lhs: this.through,
                lhs_field: this.through_remote_field,
                rhs: this.related,
                rhs_field: this.related._schema.get_field_by_name('pk'),
                join_type: this.join_type()
            }
        ];
    };
    ManyToMany.prototype.connect = function() {
        var BaseModel, file, model, models, target, _ref, _ref2;
        if (typeof this.related === 'string') {
            _ref = this.related.split('#'), file = _ref[0], target = _ref[1];
            file = require(file);
            this.related = file[target];
        }
        if (this.through) {
            if (typeof this.through === 'string') {
                _ref2 = this.through.split('#'), file = _ref2[0], target = _ref2[1];
                file = require(file);
                this.related = file[target];
            }
        } else {
            models = require('./models');
            model = this.model.scope.create("" + this.model._meta.name + "_" + this.name);
            model.schema({
                id: models.AutoField({}),
                from: models.ForeignKey(this.model, {}),
                to: models.ForeignKey(this.related, {})
            });
            BaseModel = require('./models').BaseModel;
            BaseModel.lock(model);
            this.through = model;
        }
        return process.nextTick(__bind(function() {
            var field, from_field, mgr_foreign, mgr_local, through, to_field, _i, _len, _ref3;
            from_field = to_field = null;
            _ref3 = this.through._schema.fields;
            for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
                field = _ref3[_i];
                if (field.related === this.model) {
                    from_field = field;
                } else if (field.related === this.related) {
                    to_field = field;
                }
            }
            if (!(from_field && to_field)) {
                throw new Error("Could not create M2M for " + this.model._meta.name + "." + this.name);
            }
            this.through_local_field = from_field;
            this.through_remote_field = to_field;
            mgr_local = new Manager(this.related);
            mgr_foreign = new Manager(this.model);
            mgr_local.start_query = function() {
                var base, filter;
                base = Manager.prototype.start_query.call(mgr_local);
                filter = {};
                filter["" + (to_field.get_related_name()) + "__" + from_field.name + "__pk__exact"] = this.instance.pk;
                return base.filter(filter);
            };
            through = this.through;
            mgr_local.add = function(item) {
                var kwargs;
                kwargs = {};
                kwargs[from_field.name] = this.instance;
                kwargs[to_field.name] = item;
                return through.objects.get_or_create(kwargs);
            };
            mgr_local.remove = function(item) {
                var kwargs;
                kwargs = {};
                kwargs[from_field.name] = this.instance;
                kwargs[to_field.name] = item;
                return through.objects.filter(kwargs)["delete"]();
            };
            mgr_foreign.start_query = function() {
                var base, filter;
                base = Manager.prototype.start_query.call(mgr_foreign);
                filter = {};
                filter["" + (from_field.get_related_name()) + "__" + to_field.name + "__pk__exact"] = this.instance.pk;
                return base.filter(filter);
            };
            mgr_foreign.add = function(item) {
                var kwargs;
                kwargs = {};
                kwargs[to_field.name] = this.instance;
                kwargs[from_field.name] = item;
                return through.objects.get_or_create(kwargs);
            };
            mgr_foreign.remove = function(item) {
                var kwargs;
                kwargs = {};
                kwargs[to_field.name] = this.instance;
                kwargs[from_field.name] = item;
                return through.objects.filter(kwargs)["delete"]();
            };
            this.model.register_manager(this.name, mgr_local);
            this.related.register_manager(this.get_related_name(), mgr_foreign);
            this.model._schema.register_related_field(this.name, this);
            return this.related._schema.register_related_field(new ReverseRelation(this.get_related_name(), this.to_field, this));
        }, this));
    };
    Count = function(alias) {
        this.alias = alias;
        return this;
    };
    Count.prototype = new PositiveIntegerField({});
    Count.prototype.is_decoration = true;
    Count.prototype.db_repr = function(connection) {
        return "COUNT(*) as " + (connection.quote(this.alias));
    };
    Count.prototype.db_field = function() {
        return this.alias;
    };
    module.exports = exports = {
        Count: Count,
        Field: Field,
        ForeignKey: ForeignKey,
        IntegerField: IntegerField,
        PositiveIntegerField: PositiveIntegerField,
        AutoField: AutoField,
        CharField: CharField,
        TextField: TextField,
        ManyToMany: ManyToMany,
        DateField: DateField,
        DateTimeField: DateTimeField
    };
}).call(this);
