(function() {
    var EventEmitter, Manager, QuerySet;
    var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
    EventEmitter = require('./events').EventEmitter;
    QuerySet = require('./query').QuerySet;
    Manager = function(model_fn) {
        this.model = model_fn;
        return this;
    };
    Manager.prototype.get_base_payload = function() {
        return null;
    };
    Manager.prototype.start_query = function() {
        return new QuerySet(this.model);
    };
    Manager.prototype.filter = function(kwargs) {
        var q;
        q = this.start_query();
        return q.filter(kwargs);
    };
    Manager.prototype.all = function() {
        return this.start_query();
    };
    Manager.prototype.create = function(kwargs) {
        var base, key, q, val;
        q = this.start_query();
        base = this.get_base_payload() || {};
        for (key in kwargs) {
            val = kwargs[key];
            base[key] = val;
        }
        return q.create(base);
    };
    Manager.prototype["delete"] = function() {
        return this.start_query()["delete"]();
    };
    Manager.prototype.update = function(kwargs) {
        return this.start_query().update(kwargs);
    };
    Manager.prototype.get = function(kwargs) {
        var base, ee;
        ee = new EventEmitter;
        base = this.filter(kwargs).limit(2);
        base.on('data', __bind(function(data) {
            if (data.length > 1) {
                return ee.emit('error', new this.model.MultipleObjectsReturned);
            } else if (data.length < 1) {
                return ee.emit('error', new this.model.DoesNotExist);
            } else {
                return ee.emit('data', data[0]);
            }
        }, this));
        base.on('error', function(err) {
            return ee.emit('error', err);
        });
        return ee;
    };
    Manager.prototype.get_or_create = function(kwargs) {
        var base, ee;
        ee = new EventEmitter;
        base = this.get(kwargs);
        base.on('data', function(data) {
            return ee.emit('data', data);
        });
        base.on('error', __bind(function(err) {
            var new_base;
            if (err instanceof this.model.DoesNotExist) {
                new_base = this.create(kwargs);
                new_base.on('data', function(data) {
                    return ee.emit('data', data);
                });
                return new_base.on('error', function(err) {
                    return ee.emit('error', err);
                });
            } else {
                return ee.emit('error', err);
            }
        }, this));
        return ee;
    };
    Manager.prototype.count = Manager.prototype.length = function() {
        return this.start_query().count();
    };
    exports.Manager = Manager;
}).call(this);
