(function() {
    var BaseModel, Connection, DoesNotExist, Manager, Meta, MultipleObjectsReturned, Schema, ValidationError, field, fields, name, _fn, _ref, _ref2;
    fields = require('./fields');
    _ref = require('./exceptions'), DoesNotExist = _ref.DoesNotExist, MultipleObjectsReturned = _ref.MultipleObjectsReturned, ValidationError = _ref.ValidationError;
    Manager = require('./managers').Manager;
    _ref2 = require('./schema'), Schema = _ref2.Schema, Meta = _ref2.Meta;
    Connection = require('./connection').Connection;
    BaseModel = function(name, model_fn) {
        var dne_error, mul_error;
        this.name = name;
        model_fn._meta = new Meta(name, model_fn);
        model_fn._schema = new Schema(model_fn);
        dne_error = function() {
            DoesNotExist.call(this);
            return this;
        };
        dne_error.prototype = new DoesNotExist;
        mul_error = function() {
            MultipleObjectsReturned.call(this);
            return this;
        };
        mul_error.prototype = new MultipleObjectsReturned;
        model_fn.DoesNotExist = dne_error;
        model_fn.MultipleObjectsReturned = mul_error;
        model_fn.schema = function(schema) {
            return BaseModel.set_schema(model_fn, schema);
        };
        model_fn.meta = function(meta) {
            return BaseModel.set_meta(model_fn, meta);
        };
        this.__mro__ = this.__mro__ ? [model_fn].concat(this.__mro__) : [model_fn];
        this.constructor = model_fn;
        return this;
    };
    BaseModel.lock = function(model_fn) {
        if (model_fn.__locked__) {
            return;
        }
        model_fn.schema = function() {
            throw new Error("" + model_fn._meta.name + "'s schema is locked!");
        };
        model_fn.meta = function() {
            throw new Error("" + model_fn._meta.name + "'s meta is locked!");
        };
        this.compile(model_fn);
        return model_fn.__locked__ = true;
    };
    BaseModel.set_meta = function(model_fn, meta) {
        var key, value, _results;
        _results = [];
        for (key in meta) {
            value = meta[key];
            _results.push(model_fn._meta.set(key, value));
        }
        return _results;
    };
    BaseModel.set_schema = function(model_fn, schema) {
        var key, value, _results;
        _results = [];
        for (key in schema) {
            value = schema[key];
            _results.push(model_fn._schema.set(key, value));
        }
        return _results;
    };
    BaseModel.compile = function(model_fn) {
        var id_field;
        if (!model_fn._schema.has_primary_key) {
            id_field = model_fn._schema.set('id', new fields.AutoField({}));
            model_fn._schema.alias('pk', id_field);
        }
        return model_fn._schema.connect_related();
    };
    BaseModel.create_manager = function(model_fn) {
        model_fn._default_manager = new Manager(model_fn);
        return model_fn.objects = model_fn.objects ? new model_fn.objects(model_fn) : model_fn._default_manager;
    };
    BaseModel.prototype.assign = function(kwargs) {
        var field, key, schema, val, _results;
        schema = this.constructor._schema;
        _results = [];
        for (key in kwargs) {
            val = kwargs[key];
            field = (schema.get_field_by_name(key)) || (schema.get_field_by_db_name(key));
            _results.push((function() {
                if (!field) {
                    throw new ValidationError("" + key + " is not a valid field for " + this.constructor._meta.name);
                } else {
                    return field.apply_value(this, val);
                }
            }).call(this));
        }
        return _results;
    };
    BaseModel.prototype.value_dict = function() {
        var field_name, output, _i, _len, _ref3;
        output = {};
        _ref3 = this.constructor._schema.get_field_names();
        for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
            field_name = _ref3[_i];
            if (this[field_name] !== void 0) {
                output[field_name] = this[field_name];
            }
        }
        return output;
    };
    BaseModel.prototype.save = function() {
        if (this.pk) {
            return this.constructor.objects.filter({
                pk: this.pk
            }).update(this.value_dict());
        } else {
            return this.constructor.objects.create(this.value_dict());
        }
    };
    BaseModel.prototype["delete"] = function() {
        if (this.pk) {
            return this.constructor.objects.filter({
                pk: this.pk
            })["delete"]();
        } else {
            throw new Error("Can't delete object!");
        }
    };
    exports.namespace = function(name, do_fn) {
        var Scope, scope;
        Scope = require('./scope').Scope;
        scope = new Scope(name);
        scope.execute(do_fn);
        return scope;
    };
    exports.BaseModel = BaseModel;
    exports.Connection = Connection;
    exports.configure = function(name, connection_data) {
        return Connection.register(name, connection_data);
    };
    exports.set_prefix = function(prefix) {
        return Meta.set_prefix(prefix);
    };
    _fn = function(name, field) {
        return exports[name] = function(a, b) {
            var ret;
            if (!(a != null)) {
                a = {};
            }
            if (!(b != null)) {
                b = {};
            }
            ret = new field(a, b);
            ret.type = name;
            ret.original_arguments = [].slice.call(arguments);
            return ret;
        };
    };
    for (name in fields) {
        field = fields[name];
        _fn(name, field);
    }
}).call(this);
