(function() {
    var AND, Connection, DELETE, EventEmitter, INSERT, NOT, OR, QCons, Query, QueryLeaf, QuerySet, SELECT, UPDATE, clone, valid_comparisons, _ref;
    var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice, __indexOf = Array.prototype.indexOf || function(item) {
      for (var i = 0, l = this.length; i < l; i++) {
        if (this[i] === item) return i;
      }
      return -1;
    };
    EventEmitter = require('./events').EventEmitter;
    QCons = require('./qcons').QCons;
    Connection = require('./connection').Connection;
    _ref = require('./constants'), AND = _ref.AND, OR = _ref.OR, NOT = _ref.NOT, SELECT = _ref.SELECT, INSERT = _ref.INSERT, UPDATE = _ref.UPDATE, DELETE = _ref.DELETE;
    valid_comparisons = ['exact', 'iexact', 'lt', 'gt', 'lte', 'gte', 'in', 'isnull', 'contains', 'startswith', 'endswith', 'icontains', 'istartswith', 'iendswith', 'range', 'year', 'month', 'day', 'week_day', 'regex'];
    clone = function(obj, props) {
        var key, out, val;
        out = Object.create(obj);
        for (key in props) {
            val = props[key];
            out[key] = val;
        }
        return out;
    };
    Query = function(conn, children) {
        this.conn = conn;
        this.children = children;
        return this;
    };
    Query.prototype.op_and = function(what) {
        return new Query(AND, [this, what]);
    };
    Query.prototype.op_or = function(what) {
        return new Query(OR, [this, what]);
    };
    Query.prototype.op_not = function() {
        return new Query(NOT, [this]);
    };
    Query.prototype.compile = function(qcons) {
        var bits;
        bits = this.children.map(function(child) {
            return child.compile(qcons);
        });
        if (this.conn === NOT) {
            return this.conn + ("(" + (bits.join(AND)) + ")");
        } else {
            return "(" + (bits.join(this.conn.toString())) + ")";
        }
    };
    QueryLeaf = function(field, value, cmp) {
        var _ref2;
        _ref2 = [field, value, cmp], this.field = _ref2[0], this.value = _ref2[1], this.cmp = _ref2[2];
        return this;
    };
    QueryLeaf.prototype.compile = function(qcons) {
        return qcons.compile_where_clause(this.field, qcons.queryset.connection.comparison(this.cmp), this.value);
    };
    QuerySet = EventEmitter.subclass(function(model) {
        this.using_connection = 'default';
        this.model = model;
        this.timeout = setTimeout(__bind(function() {
            return this.execute();
        }, this), 0);
        this.query = null;
        this.payload = null;
        this._limit = null;
        this.order_by = this.model._meta.order_by || null;
        this._create = this._delete = false;
        this._errored = null;
        this._fields = null;
        this._ready_count = 0;
        return this;
    });
    QuerySet.prototype.ready = function() {
        return this._ready_count === 0;
    };
    QuerySet.prototype.execute = function() {
        var interval, qcons, ready;
        qcons = new QCons(this);
        if (!this.connection) {
            this.connection = Connection.get_connection(this.using_connection);
        }
        ready = __bind(function() {
            var ee, sql;
            qcons.set_mode(SELECT);
            if (this.payload || this._create) {
                if (this._create) {
                    qcons.set_mode(INSERT);
                } else {
                    qcons.set_mode(UPDATE);
                }
                qcons.add_payload(this.payload);
            } else if (this._delete) {
                qcons.set_mode(DELETE);
            }
            qcons.set_fields(this._fields || this.model._schema.real_fields());
            qcons.set_limit(this._limit);
            if (this.query) {
                qcons.set_where(this.query.compile(qcons));
            }
            if (this.order_by) {
                this.order_by.forEach(function(field_name) {
                    return qcons.add_ordering(field_name);
                });
            }
            sql = qcons.compile();
            ee = this.connection.execute(sql, qcons.prepared_values(), qcons.mode, this.model);
            ee.on('data', __bind(function(data) {
                return this.emit('data', qcons.coerce(data));
            }, this));
            return ee.on('error', __bind(function(error) {
                return this.emit('error', error);
            }, this));
        }, this);
        interval = setInterval(__bind(function() {
            if (this.ready()) {
                clearInterval(interval);
                if (this._errored) {
                    return this.emit('error', this._errored);
                } else {
                    try {
                        return ready();
                    } catch (err) {
                        return this.emit('error', err);
                    }
                }
            }
        }, this), 1);
        return this;
    };
    QuerySet.prototype.using = function(name) {
        this.using_connection = name;
        return this;
    };
    QuerySet.prototype.count = QuerySet.prototype.length = function(ready) {
        var Count, ee;
        Count = require('./fields').Count;
        this._fields = [new Count('count')];
        ee = new EventEmitter;
        this(function(err, data) {
            if (err) {
                return ee.emit('error', err);
            } else {
                return ee.emit('data', data[0].count);
            }
        });
        return ee;
    };
    QuerySet.prototype._select_query = function(kwargs) {
        var children, cmp, fields, key, leaf, query, val, _i, _ref2;
        if (this.model._schema.validate(kwargs)) {
            children = [];
            for (key in kwargs) {
                val = kwargs[key];
                _ref2 = key.split('__'), fields = 2 <= _ref2.length ? __slice.call(_ref2, 0, _i = _ref2.length - 1) : (_i = 0, []), cmp = _ref2[_i++];
                if (!(__indexOf.call(valid_comparisons, cmp) >= 0)) {
                    fields.push(cmp);
                    cmp = 'exact';
                }
                leaf = new QueryLeaf(fields, val, cmp);
                if (val && val.on) {
                    ++this._ready_count;
                    val.on('data', __bind(function(data) {
                        --this._ready_count;
                        return leaf.value = data;
                    }, this));
                    val.on('error', __bind(function(err) {
                        --this._ready_count;
                        return this._errored = err;
                    }, this));
                }
                children.push(leaf);
            }
            return query = new Query(AND, children);
        }
    };
    QuerySet.prototype._update_payload = function(kwargs) {
        var key, val, _results;
        if (this.model._schema.validate(kwargs, true)) {
            this.payload = this.payload ? clone(this.payload, kwargs) : kwargs;
            _results = [];
            for (key in kwargs) {
                val = kwargs[key];
                _results.push(val && val.on ? (++this._ready_count, __bind(function(key, val) {
                    val.on('data', __bind(function(data) {
                        --this._ready_count;
                        return this.payload[key] = data;
                    }, this));
                    return val.on('error', __bind(function(err) {
                        --this._ready_count;
                        return this._errored = err;
                    }, this));
                }, this)(key, val)) : void 0);
            }
            return _results;
        }
    };
    QuerySet.prototype.exclude = function(kwargs) {
        var q;
        q = this._select_query(kwargs);
        if (this.query) {
            this.query = this.query.op_and(this.query.op_not());
        } else {
            this.query = q.op_not();
        }
        return this;
    };
    QuerySet.prototype.order_by = function() {
        var ordering;
        ordering = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
        ordering.forEach(function(val) {
            if ('-' === val.charAt(0)) {
                val = val.slice(1);
            }
            return this.model._schema.validate(val);
        });
        return this.order_by = ordering;
    };
    QuerySet.prototype.limit = function(from, to) {
        var _ref2;
        _ref2 = to !== void 0 ? [from, to] : [0, from], from = _ref2[0], to = _ref2[1];
        this._limit = {
            from: from,
            to: to
        };
        return this;
    };
    QuerySet.prototype.filter = function(kwargs) {
        var q;
        q = this._select_query(kwargs);
        if (this.query) {
            this.query = this.query.op_and(q);
        } else {
            this.query = q;
        }
        return this;
    };
    QuerySet.prototype.each = function(fn) {
        return this.on('data', function(items) {
            var item, _i, _len, _results;
            _results = [];
            for (_i = 0, _len = items.length; _i < _len; _i++) {
                item = items[_i];
                _results.push(fn(item));
            }
            return _results;
        });
    };
    QuerySet.prototype.update = function(kwargs) {
        this._update_payload(kwargs);
        return this;
    };
    QuerySet.prototype["delete"] = function() {
        this._delete = true;
        return this;
    };
    QuerySet.prototype.values_list = function() {
        var ee, val, values, _i, _len;
        values = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
        for (_i = 0, _len = values.length; _i < _len; _i++) {
            val = values[_i];
            if (!this.model._schema.get_field_by_name(val)) {
                this._errored = new SchemaError("" + val + " is not a valid field on " + this.model._meta.name);
                return this;
            }
        }
        ee = new EventEmitter;
        this(function(err, data) {
            var instance, key;
            if (err) {
                return ee.emit('error', err);
            } else {
                return ee.emit('data', (function() {
                    var _j, _len2, _results;
                    _results = [];
                    for (_j = 0, _len2 = data.length; _j < _len2; _j++) {
                        instance = data[_j];
                        _results.push((function() {
                            var _k, _len3, _results2;
                            _results2 = [];
                            for (_k = 0, _len3 = values.length; _k < _len3; _k++) {
                                key = values[_k];
                                _results2.push(instance[key]);
                            }
                            return _results2;
                        })());
                    }
                    return _results;
                })());
            }
        });
        return ee;
    };
    QuerySet.prototype.flat_values_list = function() {
        var ee, values;
        values = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
        ee = new EventEmitter;
        values = this.values_list.apply(this, values);
        values(function(err, data) {
            var instance, out, value, _i, _j, _len, _len2;
            if (err) {
                return ee.emit('error', err);
            } else {
                out = [];
                for (_i = 0, _len = data.length; _i < _len; _i++) {
                    instance = data[_i];
                    for (_j = 0, _len2 = instance.length; _j < _len2; _j++) {
                        value = instance[_j];
                        out.push(value);
                    }
                }
                return ee.emit('data', out);
            }
        });
        return ee;
    };
    QuerySet.prototype.create = function(kwargs) {
        this._update_payload(kwargs);
        this._create = true;
        return this;
    };
    exports.QuerySet = QuerySet;
}).call(this);
