(function() {
    var Field, Meta, Schema, base_keys, lowercase;
    var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
    Field = require('./fields').Field;
    lowercase = String.prototype.toLowerCase.call.bind(String.prototype.toLowerCase);
    base_keys = function(kwargs) {
        return Object.keys(kwargs).map(function(key) {
            return key.split('__')[0];
        });
    };
    Schema = function(model) {
        this.fields = [];
        this.related_fields = [];
        this.aliases = {};
        this.model = model;
        this.has_primary_key = false;
        return this;
    };
    Schema.prototype.get_field_names = function() {
        var field, _i, _len, _ref, _results;
        _ref = this.fields;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            field = _ref[_i];
            if (field.db_field()) {
                _results.push(field.name);
            }
        }
        return _results;
    };
    Schema.prototype.get_field_by_db_name = function(key) {
        var field;
        return ((function() {
            var _i, _len, _ref, _results;
            _ref = this.fields;
            _results = [];
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                field = _ref[_i];
                if (field.db_field() === key) {
                    _results.push(field);
                }
            }
            return _results;
        }).call(this))[0];
    };
    Schema.prototype.get_field_by_name = function(key) {
        var field, _i, _j, _len, _len2, _ref, _ref2;
        if (this.aliases[key]) {
            return this.aliases[key];
        }
        _ref = this.related_fields;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            field = _ref[_i];
            if (field.name === key) {
                return field;
            }
        }
        _ref2 = this.fields;
        for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
            field = _ref2[_j];
            if (field.name === key) {
                return field;
            }
        }
        return;
    };
    Schema.prototype.register_related_field = function(field) {
        return this.related_fields.push(field);
    };
    Schema.prototype.set = function(key, val) {
        var field_instance;
        if (val instanceof Function) {
            val = val({});
        }
        if (!val instanceof Field) {
            throw new SchemaError("" + this.model._meta.name + "." + key + " is not an instance of Field");
        }
        field_instance = val.create_instance(key, this.model);
        if (val.primary_key) {
            this.has_primary_key = true;
            this.alias('pk', field_instance);
        }
        this.fields.push(field_instance);
        return field_instance;
    };
    Schema.prototype.alias = function(name, to_field) {
        return this.aliases[name] = to_field;
    };
    Schema.prototype.real_fields = function() {
        var field, _i, _len, _ref, _results;
        _ref = this.fields;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            field = _ref[_i];
            if (field.db_field()) {
                _results.push(field);
            }
        }
        return _results;
    };
    Schema.prototype.connect_related = function() {
        var field, _i, _len, _ref, _results;
        _ref = this.fields;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            field = _ref[_i];
            if (field.needs_connection()) {
                _results.push(field.connect());
            }
        }
        return _results;
    };
    Schema.prototype.validate = function(kwargs, strict) {
        var fields;
        if (strict == null) {
            strict = false;
        }
        kwargs = kwargs || {};
        fields = strict ? Object.keys(kwargs) : base_keys(kwargs);
        fields.forEach(__bind(function(field_name) {
            var field_instance;
            field_instance = this.get_field_by_name(field_name);
            if (!field_instance) {
                return false;
                throw new SchemaError("" + field_name + " is not a field on " + this.model._meta.name);
            }
        }, this));
        return true;
    };
    Meta = function(name, model) {
        this.model = model;
        this.name = name;
        this.db_table = Meta.get_db_table_name(name);
        return this;
    };
    Meta.set_prefix = function(prefix) {
        return this.prefix = prefix;
    };
    Meta.get_db_table_name = function(name) {
        if (this.prefix) {
            return this.prefix + '_' + lowercase(name);
        } else {
            return lowercase(name);
        }
    };
    Meta.prototype.get_table_constraints = function(conn) {
        var constraints, field, schema;
        constraints = [];
        schema = this.model._schema;
        if (this.unique_together && this.unique_together instanceof Array) {
            constraints.push("CONSTRAINT " + (conn.quote(this.unique_together.join('_') + '_unique')) + " UNIQUE (" + (((function() {
                var _i, _len, _ref, _results;
                _ref = this.unique_together;
                _results = [];
                for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                    field = _ref[_i];
                    _results.push(conn.quote(schema.get_field_by_name(field).db_field()));
                }
                return _results;
            }).call(this)).join(', ')) + ")");
        }
        return constraints;
    };
    Meta.prototype.set = function(key, val) {
        return this[key] = val;
    };
    exports.Schema = Schema;
    exports.Meta = Meta;
}).call(this);
