(function() {
    var BaseModel, Connection, Scope, set_prefix, _ref;
    var __indexOf = Array.prototype.indexOf || function(item) {
      for (var i = 0, l = this.length; i < l; i++) {
        if (this[i] === item) return i;
      }
      return -1;
    }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
    _ref = require('./models'), BaseModel = _ref.BaseModel, set_prefix = _ref.set_prefix;
    Connection = require('./connection').Connection;
    Scope = function(name) {
        this.name = name;
        this.models = {};
        this.depends_on_scopes = [];
        return this;
    };
    Scope.prototype.depend_on = function(scope) {
        var _ref2;
        if (_ref2 = !scope, __indexOf.call(this.depends_on_scopes, _ref2) >= 0) {
            return this.depends_on_scopes.push(scope);
        }
    };
    Scope.prototype.execute = function(scope_fn) {
        var repeat, seen_models;
        set_prefix(this.name);
        scope_fn(this);
        seen_models = [];
        repeat = __bind(function() {
            var model, name, _ref2;
            _ref2 = this.models;
            for (name in _ref2) {
                model = _ref2[name];
                if (!(__indexOf.call(seen_models, model) >= 0)) {
                    BaseModel.lock(model);
                    seen_models.push(model);
                }
            }
            if (seen_models.length < Object.keys(this.models)) {
                return repeat();
            }
        }, this);
        return repeat();
    };
    Scope.prototype.create = function(model_name) {
        var managers, model_fn;
        managers = {};
        model_fn = function(kwargs) {
            var key, value, _ref2;
            this._fk_cache = {};
            this.assign(kwargs);
            for (key in managers) {
                value = managers[key];
                this[key] = value;
                this[key].instance = this;
            }
            _ref2 = model_fn._schema.aliases;
            for (key in _ref2) {
                value = _ref2[key];
                Object.defineProperty(this, key, {
                    get: __bind(function() {
                        return this[value.name];
                    }, this),
                    set: __bind(function(val) {
                        return this[value.name] = val;
                    }, this)
                });
            }
            return this;
        };
        model_fn.register_manager = function(at, manager) {
            return managers[at] = manager;
        };
        model_fn.prototype = new BaseModel(model_name, model_fn);
        BaseModel.create_manager(model_fn);
        this.models[model_name] = model_fn;
        model_fn.scope = this;
        return model_fn;
    };
    Scope.prototype.db_creation = function(connection, execute, ready) {
        var constraint, data, db_field, errors, field, fields, model, name, pending_constraints, recurse, sql, table_constraints, visited, _i, _len, _ref2, _ref3, _ref4, _sql;
        if (execute == null) {
            execute = true;
        }
        if (typeof connection === 'string') {
            connection = Connection.get_connection(connection);
        }
        if (execute instanceof Function) {
            ready = execute;
            execute = true;
        }
        pending_constraints = [];
        sql = [];
        visited = [];
        _ref2 = this.models;
        for (name in _ref2) {
            model = _ref2[name];
            fields = [];
            _ref3 = model._schema.fields;
            for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
                field = _ref3[_i];
                if (field.db_field()) {
                    db_field = connection.negotiate_type(field);
                    db_field.contribute_to_table(fields, pending_constraints, visited);
                }
            }
            table_constraints = model._meta.get_table_constraints(connection) || [];
            visited.push(model);
            _sql = "CREATE TABLE " + model._meta.db_table + " (\n    " + (fields.join(', ')) + "\n    " + (table_constraints.length ? ', ' + table_constraints.join(', ') : '') + "\n);";
            sql.push(_sql);
        }
        if (pending_constraints.length) {
            sql.push(((function() {
                var _j, _len2, _results;
                _results = [];
                for (_j = 0, _len2 = pending_constraints.length; _j < _len2; _j++) {
                    constraint = pending_constraints[_j];
                    _results.push(connection.constraint(constraint));
                }
                return _results;
            })()).join(';'));
        }
        if (execute) {
            _ref4 = [[], []], errors = _ref4[0], data = _ref4[1];
            recurse = function() {
                var done, ee;
                ee = connection.execute(sql.shift(), [], null, null);
                done = function() {
                    if (sql.length) {
                        return recurse();
                    } else {
                        return ready(errors, data);
                    }
                };
                ee.on('err', function(err) {
                    return [errors.push(err), done()];
                });
                return ee.on('data', function(result) {
                    return [data.push(result), done()];
                });
            };
            if (sql.length) {
                return recurse();
            } else {
                return ready(errors, data);
            }
        } else {
            return console.log(sql);
        }
    };
    Scope.prototype.db_deletion = function(connection, execute, ready) {
        var ForeignKey, data, errors, model, models, name, recurse, scope, sql, _ref2;
        if (execute == null) {
            execute = true;
        }
        if (typeof connection === 'string') {
            connection = Connection.get_connection(connection);
        }
        if (execute instanceof Function) {
            ready = execute;
            execute = true;
        }
        models = (function() {
            var _ref2, _results;
            _ref2 = this.models;
            _results = [];
            for (name in _ref2) {
                model = _ref2[name];
                _results.push(model);
            }
            return _results;
        }).call(this);
        ForeignKey = require('./fields').ForeignKey;
        scope = this;
        sql = [];
        recurse = function(model) {
            var drop_sql, field, _i, _len, _ref2, _results;
            if (__indexOf.call(models, model) < 0) {
                return;
            }
            drop_sql = connection.drop_table(model);
            if (!(__indexOf.call(sql, drop_sql) >= 0)) {
                sql.push(drop_sql);
            }
            models.splice(models.indexOf(model), 1);
            _ref2 = model._schema.fields;
            _results = [];
            for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
                field = _ref2[_i];
                _results.push(field instanceof ForeignKey && field.related.scope === scope ? recurse(field.related) : field.through && field.through.scope === scope ? recurse(field.through) : void 0);
            }
            return _results;
        };
        while (models.length) {
            recurse(models[0]);
        }
        if (execute) {
            _ref2 = [[], []], errors = _ref2[0], data = _ref2[1];
            recurse = function() {
                var done, ee;
                ee = connection.execute(sql.shift(), [], null, null);
                done = function() {
                    if (sql.length) {
                        return recurse();
                    } else {
                        return ready(errors, data);
                    }
                };
                ee.on('err', function(err) {
                    return [errors.push(err), done()];
                });
                return ee.on('data', function(result) {
                    return [data.push(result), done()];
                });
            };
            if (sql.length) {
                return recurse();
            } else {
                return ready(errors, data);
            }
        } else {
            return console.log(sql.join(';\n') + ';\n');
        }
    };
    exports.Scope = Scope;
}).call(this);
