(function() {
  var exceptions, exports, models, test, unittest, _ref;
  _ref = require('./_utils'), unittest = _ref.unittest, test = _ref.test, models = _ref.models, exceptions = _ref.exceptions;
  module.exports = exports = {
    'test autofield pk': unittest(models.namespace('custom_pk', function(ns) {
      var Model;
      Model = ns.create('Model');
      return Model.schema({
        my_own_pk: models.AutoField,
        handle: models.CharField({
          "default": function() {
            return "whatever";
          }
        })
      });
    }), test("Test that creation of a custom pk model works as expected.", function(ns, assert) {
      var creation;
      creation = ns.models.Model.objects.create({});
      return creation(assert.async(function(err, data) {
        assert.fail(err);
        assert.ok(data);
        assert.strictEqual(data.pk, data.my_own_pk);
        assert.fail(data.id);
        return assert.fail(ns.models.Model._schema.get_field_by_name('id'));
      }));
    }), test("Test that filtering based on pk returns appropriate results.", function(ns, assert) {
      var creation, expected;
      expected = 'filtering_test_' + ~~(Math.random() * 100);
      creation = ns.models.Model.objects.create({});
      return creation(assert.async(function(err, data) {
        assert.fail(err);
        assert.ok(data);
        assert.strictEqual(data.pk, data.my_own_pk);
        return ns.models.Model.objects.get({
          pk__exact: data.my_own_pk
        })(assert.async(function(err, instance) {
          assert.fail(err);
          assert.ok(instance);
          return assert.equal(instance.handle, data.handle);
        }));
      }));
    })),
    'test non-integer pk': unittest(models.namespace('custom_nonint_pk', function(ns) {
      var FKModel, M2MModel, Model;
      Model = ns.create('Model');
      Model.schema({
        my_personal_pk: models.CharField({
          primary_key: true
        })
      });
      FKModel = ns.create('FKModel');
      FKModel.schema({
        model: models.ForeignKey(Model),
        name: models.CharField({
          "default": function() {
            return "anything really " + (~~(Math.random() * 100));
          }
        })
      });
      M2MModel = ns.create('M2MModel');
      return M2MModel.schema({
        models: models.ManyToMany(Model),
        name: models.CharField({
          "default": function() {
            return "anything really " + (~~(Math.random() * 100));
          }
        })
      });
    }), test("Test that non-integer primary key INSERT works as expected.", function(ns, assert) {
      var Model, creation, expected;
      Model = ns.models.Model;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      creation = Model.objects.create({
        my_personal_pk: expected
      });
      return creation(assert.async(function(err, data) {
        assert.fail(err);
        assert.ok(data);
        assert.equal(data.pk, expected);
        return assert.equal(data.my_personal_pk, data.pk);
      }));
    }), test("Test that non-integer primary key SELECT works as expected.", function(ns, assert) {
      var Model, done, fn, i;
      Model = ns.models.Model;
      i = 0;
      fn = function() {
        var creation, expected;
        if (i < 10) {
          expected = 'select_text_pk_' + ~~(Math.random() * 100) + '_' + (i++);
          creation = Model.objects.create({
            my_personal_pk: expected
          });
          return creation(fn);
        } else {
          return done();
        }
      };
      done = assert.async(function() {
        return Model.objects.filter({
          pk__startswith: 'select_text_'
        })(assert.async(function(err, pk_data) {
          assert.fail(err);
          assert.ok(pk_data);
          assert.equal(pk_data.length, 10);
          return Model.objects.filter({
            my_personal_pk__startswith: 'select_text_'
          })(assert.async(function(err, n_data) {
            var i, _ref2, _results;
            assert.fail(err);
            assert.ok(n_data);
            assert.equal(n_data.length, pk_data.length);
            _results = [];
            for (i = 0, _ref2 = n_data.length; 0 <= _ref2 ? i < _ref2 : i > _ref2; 0 <= _ref2 ? i++ : i--) {
              assert.equal(n_data[i].pk, pk_data[i].pk);
              assert.equal(n_data[i].my_personal_pk, pk_data[i].my_personal_pk);
              assert.equal(n_data[i].pk, pk_data[i].my_personal_pk);
              _results.push(assert.equal(n_data[i].my_personal_pk, pk_data[i].pk));
            }
            return _results;
          }));
        }));
      });
      return fn();
    }), test("Test that non-integer primary key UPDATE works as expected.", function(ns, assert) {
      var Model, creation, expected, expected2;
      Model = ns.models.Model;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      expected2 = 'yeah_text_pk_' + ~~(Math.random() * 100);
      creation = Model.objects.create({
        my_personal_pk: expected
      });
      return creation(assert.async(function(err, data) {
        var update;
        assert.fail(err);
        assert.ok(data);
        assert.equal(data.pk, expected);
        update = Model.objects.filter({
          pk: data.pk
        }).update({
          pk: expected2
        });
        return update(assert.async(function(err, data) {
          assert.fail(err);
          return Model.objects.filter({
            pk: expected2
          })(assert.async(function(err, data) {
            assert.fail(err);
            assert.equal(data.length, 1);
            return assert.equal(data[0].pk, expected2);
          }));
        }));
      }));
    }), test("Test that non-integer primary key DELETE works as expected.", function(ns, assert) {
      var Model, creation, expected, expected2;
      Model = ns.models.Model;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      expected2 = 'yeah_text_pk_' + ~~(Math.random() * 100);
      creation = Model.objects.create({
        my_personal_pk: expected
      });
      return creation(assert.async(function(err, data) {
        assert.fail(err);
        assert.ok(data);
        assert.equal(data.pk, expected);
        return Model.objects.filter({
          pk: data.pk
        })["delete"]()(assert.async(function(err) {
          assert.fail(err);
          return Model.objects.filter({
            pk: data.pk
          })(assert.async(function(err, data) {
            assert.fail(err);
            return assert.equal(data.length, 0);
          }));
        }));
      }));
    }), test("Test that non-integer primary key instance DELETE works as expected.", function(ns, assert) {
      var Model, creation, expected, expected2;
      Model = ns.models.Model;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      expected2 = 'yeah_text_pk_' + ~~(Math.random() * 100);
      creation = Model.objects.create({
        my_personal_pk: expected
      });
      return creation(assert.async(function(err, data) {
        assert.fail(err);
        assert.ok(data);
        assert.equal(data.pk, expected);
        return data["delete"]()(assert.async(function(err) {
          assert.fail(err);
          return Model.objects.filter({
            pk: data.pk
          })(assert.async(function(err, data) {
            assert.fail(err);
            return assert.equal(data.length, 0);
          }));
        }));
      }));
    }), test("Test that foreign key relations work as expected.", function(ns, assert) {
      var FKModel, Model, expected, _ref2;
      _ref2 = ns.models, Model = _ref2.Model, FKModel = _ref2.FKModel;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      return FKModel.objects.create({
        model: Model.objects.create({
          pk: expected
        })
      })(assert.async(function(err, fkdata) {
        assert.fail(err);
        assert.ok(fkdata);
        return fkdata.model()(assert.async(function(err, data) {
          assert.fail(err);
          assert.equal(data.pk, expected);
          return data.fkmodel_set.all()(assert.async(function(err, data) {
            assert.fail(err);
            assert.ok(data);
            assert.equal(data.length, 1);
            return assert.equal(fkdata.pk, data[0].pk);
          }));
        }));
      }));
    }), test("Test that many to many relations work as expected.", function(ns, assert) {
      var M2MModel, Model, creation, expected, _ref2;
      _ref2 = ns.models, Model = _ref2.Model, M2MModel = _ref2.M2MModel;
      expected = 'text_pk_' + ~~(Math.random() * 100);
      creation = Model.objects.create({
        my_personal_pk: expected
      });
      return creation(assert.async(function(err, model) {
        assert.fail(err);
        assert.ok(model);
        assert.equal(model.pk, expected);
        return M2MModel.objects.create({
          name: expected + '_m2m'
        })(assert.async(function(err, m2m) {
          assert.fail(err);
          assert.ok(m2m);
          return m2m.models.add(model)(assert.async(function(err, rel) {
            assert.fail(err);
            model.m2mmodel_set.filter({
              name: expected + '_m2m',
              pk: m2m.pk
            })(assert.async(function(err, m2mlist) {
              assert.fail(err);
              assert.equal(m2mlist.length, 1);
              return assert.equal(m2mlist[0].pk, m2m.pk);
            }));
            return m2m.models.filter({
              pk: expected
            })(assert.async(function(err, modellist) {
              assert.fail(err);
              assert.equal(modellist.length, 1);
              assert.equal(modellist[0].pk, model.pk);
              return assert.equal(model.pk, expected);
            }));
          }));
        }));
      }));
    }))
  };
}).call(this);
