(function() {
  var Many, Model, Related, exceptions, exports, models, test, unit, unittest, _ref, _ref2;
  Model = require('./fixtures/models').Model;
  _ref = require('./fixtures/related'), Related = _ref.Related, Many = _ref.Many;
  _ref2 = require('./_utils'), models = _ref2.models, exceptions = _ref2.exceptions, unittest = _ref2.unittest, unit = _ref2.unit, test = _ref2.test;
  module.exports = exports = {
    'test basic api': unit({}, function(assert) {
      'Test that filters are functions.';      var filter;
      filter = Model.objects.all();
      assert.ok(filter.call != null);
      assert.ok(filter.apply != null);
      assert.isInstance(filter, Function);
      return assert.doesNotThrow(Error, function() {
        return filter();
      });
    }, function(assert) {
      "Test that filters are event emitters.";      var expected, filter;
      filter = Model.objects.all();
      assert.ok(filter.on != null);
      assert.ok(filter.emit != null);
      expected = Math.random();
      filter.on('randomthing', assert.async(function(anything) {
        return assert.equal(anything, expected);
      }));
      return filter.emit('randomthing', expected);
    }, function(assert) {
      "Test that filter chaining results in the same filter.";      var filter;
      filter = Model.objects.all();
      return assert.strictEqual(filter, filter.filter({
        anything: 'random'
      }));
    }, function(assert) {
      "Test that invalid fields result in an error being emitted.";      var filter;
      filter = Model.objects.filter({
        dne: Math.random()
      });
      return filter(assert.async(function(err) {
        return assert.isInstance(err, exceptions.SchemaError);
      }));
    }, function(assert) {
      "Test that an invalid lookup parameter results in an error being emitted.";      var filter;
      filter = Model.objects.filter({
        anything: null
      });
      return filter(assert.async(function(err) {
        return assert.isInstance(err, exceptions.ValidationError);
      }));
    }, function(assert) {
      "Test that exact filtering returns results as expected.";      var creation;
      creation = Model.objects.create({
        anything: 'okay',
        validated: 'alright'
      });
      return creation(assert.async(function(err, data) {
        var correct, wrong;
        assert.fail(err);
        correct = Model.objects.filter({
          anything: 'okay'
        });
        wrong = Model.objects.filter({
          anything: 'not okay'
        });
        correct(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          assert.isInstance(rows[0], Model);
          return assert.fail(err);
        }));
        return wrong(assert.async(function(err, rows) {
          assert.fail(err);
          return assert.equal(rows.length, 0);
        }));
      }));
    }, function(assert) {
      "Test that creation with invalid parameters emits a ValidationError.";      var creation, creation2;
      creation = Model.objects.create({
        anything: 'okay',
        validated: 'anything?!?'
      });
      creation(assert.async(function(err, data) {
        assert.isInstance(err, exceptions.ValidationError);
        return assert.fail(data);
      }));
      creation2 = Model.objects.create({
        validated: 'okay'
      });
      return creation2(assert.async(function(err, data) {
        assert.isInstance(err, exceptions.ValidationError);
        return assert.fail(data);
      }));
    }, function(assert) {
      "Test that exclude works as expected.";      var creation, random_name;
      random_name = "random_" + (~~(Math.random() * 100));
      creation = Model.objects.create({
        anything: random_name,
        validated: 'okay'
      });
      return creation(assert.async(function(err, data) {
        var exclusion, pk;
        assert.fail(err);
        assert.ok(data);
        pk = data.pk;
        exclusion = Model.objects.all().exclude({
          pk: pk
        });
        return exclusion(assert.async(function(err, data) {
          var instance;
          assert.fail(err);
          assert.ok(data);
          return assert.strictEqual(((function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = data.length; _i < _len; _i++) {
              instance = data[_i];
              if (instance.pk === pk) {
                _results.push(instance);
              }
            }
            return _results;
          })()).length, 0);
        }));
      }));
    }),
    'test filtering': unit({}, function(assert) {
      "Test that exact grabs the exact model specific.";      var creation, random_name;
      random_name = "random_exact_test_" + (~~(Math.random() * 100));
      creation = Model.objects.create({
        anything: random_name,
        validated: 'okay'
      });
      return creation(assert.async(function(err, data) {
        var pk;
        assert.fail(err);
        pk = data.pk;
        return Model.objects.filter({
          anything__exact: random_name
        })(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          return assert.equal(rows[0].pk, pk);
        }));
      }));
    }, function(assert) {
      "Test that contains grabs the appropriate model.";      var creation, random_name;
      random_name = "random_contains_test_" + (~~(Math.random() * 100));
      creation = Model.objects.create({
        anything: random_name,
        validated: 'okay'
      });
      return creation(assert.async(function(err, data) {
        var pk;
        assert.fail(err);
        pk = data.pk;
        return Model.objects.filter({
          anything__contains: '_contains_'
        })(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          return assert.equal(rows[0].pk, pk);
        }));
      }));
    }, function(assert) {
      "Test that `in` returns the appropriate model(s).";      var creation, random_name;
      random_name = "random_contains_test_" + (~~(Math.random() * 100));
      creation = Model.objects.create({
        anything: random_name,
        validated: 'okay'
      });
      return creation(assert.async(function(err, data) {
        var pk;
        assert.fail(err);
        pk = data.pk;
        Model.objects.filter({
          pk__in: [pk, 1000]
        })(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          return assert.equal(rows[0].pk, pk);
        }));
        return Model.objects.filter({
          pk__in: Model.objects.filter({
            anything: random_name
          }).flat_values_list('pk')
        })(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          return assert.equal(rows[0].pk, pk);
        }));
      }));
    }, function(assert) {
      "Test that `gt` returns rows greater than the current number.";      var creation1;
      creation1 = Model.objects.create({
        anything: 'lol',
        validated: 'okay'
      });
      return creation1(assert.async(function(err, data) {
        var creation2, pk;
        assert.fail(err);
        pk = data.pk;
        creation2 = Model.objects.create({
          anything: 'lol2',
          validated: 'okay'
        });
        return creation2(assert.async(function(err, data) {
          var pk2;
          assert.fail(err);
          pk2 = data.pk;
          return Model.objects.filter({
            pk__gt: pk
          })(assert.async(function(err, data) {
            var model;
            assert.fail(err);
            assert.equal(((function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = data.length; _i < _len; _i++) {
                model = data[_i];
                if (model.pk === pk2) {
                  _results.push(model.pk);
                }
              }
              return _results;
            })()).length, 1);
            return assert.equal(((function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = data.length; _i < _len; _i++) {
                model = data[_i];
                if (model.pk === pk) {
                  _results.push(model.pk);
                }
              }
              return _results;
            })()).length, 0);
          }));
        }));
      }));
    }, function(assert) {
      "Test that `gte` returns rows greater or equal than the current number.";      var creation1;
      creation1 = Model.objects.create({
        anything: 'lol',
        validated: 'okay'
      });
      return creation1(assert.async(function(err, data) {
        var creation2, pk;
        assert.fail(err);
        pk = data.pk;
        creation2 = Model.objects.create({
          anything: 'lol2',
          validated: 'okay'
        });
        return creation2(assert.async(function(err, data) {
          var pk2;
          assert.fail(err);
          pk2 = data.pk;
          return Model.objects.filter({
            pk__gte: pk
          })(assert.async(function(err, data) {
            var model;
            assert.fail(err);
            assert.equal(data.length, 2);
            assert.equal(((function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = data.length; _i < _len; _i++) {
                model = data[_i];
                if (model.pk === pk2) {
                  _results.push(model.pk);
                }
              }
              return _results;
            })()).length, 1);
            return assert.equal(((function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = data.length; _i < _len; _i++) {
                model = data[_i];
                if (model.pk === pk) {
                  _results.push(model.pk);
                }
              }
              return _results;
            })()).length, 1);
          }));
        }));
      }));
    }, function(assert) {
      "Test that `lt` returns rows lesser than the current number.";      return Model.objects["delete"]()(assert.async(function(err, data) {
        var creation1;
        assert.fail(err);
        creation1 = Model.objects.create({
          anything: 'lol',
          validated: 'okay'
        });
        return creation1(assert.async(function(err, data) {
          var creation2, pk;
          assert.fail(err);
          pk = data.pk;
          creation2 = Model.objects.create({
            anything: 'lol2',
            validated: 'okay'
          });
          return creation2(assert.async(function(err, data) {
            var pk2;
            assert.fail(err);
            pk2 = data.pk;
            return Model.objects.filter({
              pk__lt: pk2
            })(assert.async(function(err, data) {
              var model;
              assert.fail(err);
              assert.equal(data.length, 1);
              assert.equal(((function() {
                var _i, _len, _results;
                _results = [];
                for (_i = 0, _len = data.length; _i < _len; _i++) {
                  model = data[_i];
                  if (model.pk === pk2) {
                    _results.push(model.pk);
                  }
                }
                return _results;
              })()).length, 0);
              return assert.equal(((function() {
                var _i, _len, _results;
                _results = [];
                for (_i = 0, _len = data.length; _i < _len; _i++) {
                  model = data[_i];
                  if (model.pk === pk) {
                    _results.push(model.pk);
                  }
                }
                return _results;
              })()).length, 1);
            }));
          }));
        }));
      }));
    }, function(assert) {
      "Test that `lte` returns rows lesser than the current number.";      return Model.objects["delete"]()(assert.async(function(err, data) {
        var creation1;
        assert.fail(err);
        creation1 = Model.objects.create({
          anything: 'lol',
          validated: 'okay'
        });
        return creation1(assert.async(function(err, data) {
          var creation2, pk;
          assert.fail(err);
          pk = data.pk;
          creation2 = Model.objects.create({
            anything: 'lol2',
            validated: 'okay'
          });
          return creation2(assert.async(function(err, data) {
            var pk2;
            assert.fail(err);
            pk2 = data.pk;
            return Model.objects.filter({
              pk__lte: pk2
            })(assert.async(function(err, data) {
              var model;
              assert.fail(err);
              assert.equal(data.length, 2);
              assert.equal(((function() {
                var _i, _len, _results;
                _results = [];
                for (_i = 0, _len = data.length; _i < _len; _i++) {
                  model = data[_i];
                  if (model.pk === pk2) {
                    _results.push(model.pk);
                  }
                }
                return _results;
              })()).length, 1);
              return assert.equal(((function() {
                var _i, _len, _results;
                _results = [];
                for (_i = 0, _len = data.length; _i < _len; _i++) {
                  model = data[_i];
                  if (model.pk === pk) {
                    _results.push(model.pk);
                  }
                }
                return _results;
              })()).length, 1);
            }));
          }));
        }));
      }));
    }, function(assert) {
      "Test that `startswith` returns rows with fields that start with the value.";      var random_name, random_name_start, rex;
      random_name_start = "test_startswith_" + (~~(100 * Math.random())) + "_";
      random_name = "" + random_name_start + (~~(100 * Math.random()));
      rex = new RegExp('^' + random_name_start);
      return Model.objects.create({
        anything: random_name,
        validated: 'okay'
      })(assert.async(function(err) {
        assert.fail(err);
        return Model.objects.filter({
          anything__startswith: random_name_start
        })(assert.async(function(err, instances) {
          assert.fail(err);
          assert.equal(instances.length, 1);
          return assert.ok(rex.test(instances[0].anything));
        }));
      }));
    }, function(assert) {
      "Test that `endswith` returns rows with fields that end with the value.";      var random_name, random_name_end, rex;
      random_name_end = "_test_endswith_" + (~~(100 * Math.random()));
      random_name = "" + (~~(100 * Math.random())) + random_name_end;
      rex = new RegExp(random_name_end + '$');
      return Model.objects.create({
        anything: random_name,
        validated: 'okay'
      })(assert.async(function(err) {
        assert.fail(err);
        return Model.objects.filter({
          anything__endswith: random_name_end
        })(assert.async(function(err, instances) {
          assert.fail(err);
          assert.equal(instances.length, 1);
          return assert.ok(rex.test(instances[0].anything));
        }));
      }));
    }, function(assert) {
      "Assert that range works.";      var creation, random_name;
      random_name = "random_range_test_" + (~~(Math.random() * 100));
      creation = Model.objects.create({
        anything: random_name,
        validated: 'okay'
      });
      return creation(assert.async(function(err, data) {
        var pk;
        assert.fail(err);
        pk = data.pk;
        Model.objects.filter({
          pk__range: [pk, pk + 1]
        })(assert.async(function(err, rows) {
          assert.equal(rows.length, 1);
          return assert.equal(rows[0].pk, pk);
        }));
        return Model.objects.filter({
          pk__range: [pk + 1, pk + 20]
        })(assert.async(function(err, rows) {
          return assert.equal(rows.length, 0);
        }));
      }));
    }),
    'test foreign key filtering api': unit({}, function(assert) {
      'Test that filtering on local relations works as expected.';      var creation, expected_anything;
      expected_anything = "related_anything_" + (~~(100 * Math.random()));
      creation = Related.objects.create({
        model: Model.objects.create({
          anything: expected_anything,
          validated: 'whatever'
        })
      });
      return creation(assert.async(function(err, related_instance) {
        assert.fail(err);
        return Related.objects.filter({
          model__anything: expected_anything
        })(assert.async(function(err, related_list) {
          var related;
          assert.fail(err);
          related = related_list[0];
          assert.equal(related.pk, related_instance.pk);
          return related.model()(assert.async(function(err, model_lhs) {
            return related_instance.model()(assert.async(function(err, model_rhs) {
              assert.equal(model_lhs.pk, model_rhs.pk);
              assert.equal(model_lhs.anything, expected_anything);
              return assert.equal(model_rhs.anything, expected_anything);
            }));
          }));
        }));
      }));
    }, function(assert) {
      'Test that filtering on reverse relations works as expected.';      var creation, expected_anything;
      expected_anything = "related_anything_" + (~~(100 * Math.random()));
      creation = Related.objects.create({
        model: Model.objects.create({
          anything: expected_anything,
          validated: 'whatever'
        })
      });
      return creation(assert.async(function(err, related_instance) {
        assert.fail(err);
        assert.ok(related_instance);
        return related_instance.model()(assert.async(function(err, model) {
          assert.fail(err);
          assert.ok(model);
          return Model.objects.filter({
            related_set__pk: related_instance.pk
          })(assert.async(function(err, data) {
            assert.fail(err);
            assert.ok(data);
            assert.equal(data.length, 1);
            return assert.equal(data[0].pk, model.pk);
          }));
        }));
      }));
    }, function(assert) {
      'Test that filtering back onto the originating model works as expected.';      var creation, expected_anything;
      expected_anything = "related_anything_" + (~~(100 * Math.random()));
      creation = Related.objects.create({
        model: Model.objects.create({
          anything: expected_anything,
          validated: 'whatever'
        })
      });
      return creation(assert.async(function(err, related_instance) {
        assert.fail(err);
        assert.ok(related_instance);
        return related_instance.model()(assert.async(function(err, model) {
          assert.fail(err);
          assert.ok(model);
          return Model.objects.filter({
            related_set__model__pk: model.pk
          })(assert.async(function(err, data) {
            assert.fail(err);
            assert.ok(data);
            assert.equal(data.length, 1);
            return assert.equal(data[0].pk, model.pk);
          }));
        }));
      }));
    }, function(assert) {
      'Test that filtering on M2M relations works as expected.';      var creation;
      creation = Related.objects.create({
        model: Model.objects.create({
          anything: 'lolwut',
          validated: 'whatever'
        })
      });
      return creation(assert.async(function(err, related) {
        assert.fail(err);
        assert.ok(related);
        return related.model()(assert.async(function(err, model) {
          assert.fail(err);
          assert.ok(model);
          creation = Many.objects.create({});
          return creation(assert.async(function(err, many) {
            var addition;
            assert.fail(err);
            assert.ok(many);
            addition = many.related.add(related);
            return addition(assert.async(function(err, data) {
              return Many.objects.filter({
                related__model__pk: model.pk
              })(assert.async(function(err, rows) {
                assert.fail(err);
                assert.ok(rows);
                assert.equal(rows.length, 1);
                return assert.equal(rows[0].pk, many.pk);
              }));
            }));
          }));
        }));
      }));
    }, function(assert) {
      'Test that filtering on reverse M2M relations works as expected.';      var creation;
      creation = Related.objects.create({
        model: Model.objects.create({
          anything: 'lolwut',
          validated: 'whatever'
        })
      });
      return creation(assert.async(function(err, related) {
        assert.fail(err);
        assert.ok(related);
        return related.model()(assert.async(function(err, model) {
          assert.fail(err);
          assert.ok(model);
          creation = Many.objects.create({});
          return creation(assert.async(function(err, many) {
            var addition;
            assert.fail(err);
            assert.ok(many);
            addition = many.related.add(related);
            return addition(assert.async(function(err, data) {
              return Related.objects.filter({
                manytomany_set__pk: many.pk
              })(assert.async(function(err, rows) {
                assert.fail(err);
                assert.ok(rows);
                assert.equal(rows.length, 1);
                return assert.equal(rows[0].pk, related.pk);
              }));
            }));
          }));
        }));
      }));
    }),
    'test of count': unittest(models.namespace('count', function(ns) {
      var Countable;
      Countable = ns.create('Countable');
      return Countable.schema({
        some_random_value: models.CharField({
          "default": function() {
            return 'lol';
          }
        })
      });
    }), test("Test that count works as expected", function(ns, assert) {
      var Countable, expected, i, num, ready, _results;
      num = ~~(Math.random() * 10) + 1;
      expected = num;
      Countable = ns.models.Countable;
      ready = assert.async(function(err, data) {
        assert.fail(err);
        return Countable.objects.all().count()(assert.async(function(err, data) {
          assert.fail(err);
          assert.ok(data);
          return assert.equal(data, expected);
        }));
      });
      _results = [];
      for (i = 0; 0 <= num ? i < num : i > num; 0 <= num ? i++ : i--) {
        _results.push(Countable.objects.create({})(function(err, data) {
          assert.fail(err);
          --num;
          if (num === 0) {
            return ready();
          }
        }));
      }
      return _results;
    }), test("Test that count works on filtered objects", function(ns, assert) {
      var Countable, expected, finish, num, ready, recurse;
      num = ~~(Math.random() * 10) + 10;
      expected = 0;
      Countable = ns.models.Countable;
      finish = assert.async(function() {
        return typeof lol !== "undefined" && lol !== null;
      });
      ready = function(err, data) {
        assert.fail(err);
        finish();
        return Countable.objects.filter({
          some_random_value: 'okay'
        }).count()(assert.async(function(err, data) {
          assert.fail(err);
          assert.ok(data);
          return assert.equal(data, expected);
        }));
      };
      recurse = function(err, data) {
        var creation;
        --num;
        if (num === 0) {
          return ready();
        } else {
          assert.fail(err);
          creation = {
            some_random_value: 'blah'
          };
          if (Math.random() > 0.4) {
            ++expected;
            creation = {
              some_random_value: 'okay'
            };
          }
          return Countable.objects.create(creation)(assert.async(recurse));
        }
      };
      return recurse();
    }))
  };
}).call(this);
