(function() {
  var exec, export_module, models, platoon, testing_backend;
  models = require('../').models;
  exec = require('child_process').exec;
  platoon = require('platoon');
  testing_backend = process.env.TESTING_BACKEND || 'postgres';
  models.configure('default', {
    sqlite: {
      backend: 'ormnomnom/lib/backends/sqlite',
      name: 'test_ormnomnom.db'
    },
    mysql: {
      backend: 'ormnomnom/lib/backends/mysql',
      name: 'test_ormnomnom'
    },
    postgres: {
      backend: 'ormnomnom/lib/backends/postgres',
      name: 'test_ormnomnom'
    }
  }[testing_backend]);
  export_module = function(module_name) {
    var key, mod, val, _results;
    mod = require('./' + module_name);
    _results = [];
    for (key in mod) {
      val = mod[key];
      _results.push(exports["" + module_name + ": " + key] = val);
    }
    return _results;
  };
  if (testing_backend === 'postgres') {
    platoon.setBeforeStart(function(ready) {
      return exec('createdb test_ormnomnom', function() {
        var ns;
        ns = require('./fixtures/models').ns;
        return ns.db_creation('default', true, function() {
          ns = require('./fixtures/related').ns;
          return ns.db_creation('default', true, function() {
            return ready();
          });
        });
      });
    });
    platoon.setBeforeFinish(function(ready) {
      var connection;
      connection = models.Connection.get_connection('default');
      return connection.close(function() {
        return exec('dropdb test_ormnomnom', function(err) {
          if (err) {
            return setTimeout(exec.bind({}, 'dropdb test_ormnomnom', arguments.callee), 10);
          } else {
            return ready();
          }
        });
      });
    });
  }
  if (testing_backend === 'sqlite') {
    platoon.setBeforeStart(function(ready) {
      var ns;
      ns = require('./fixtures/models').ns;
      return ns.db_creation('default', true, function(err) {
        ns = require('./fixtures/related').ns;
        return ns.db_creation('default', true, function() {
          return ready();
        });
      });
    });
    platoon.setBeforeFinish(function(ready) {
      return exec('rm test_ormnomnom.db', function() {
        return ready();
      });
    });
  }
  export_module('fields');
  export_module('models');
  export_module('filters');
}).call(this);
